from nt_toolbox.general import *
from nt_toolbox.signal import *

from matplotlib.pyplot import *
from numpy import *
from numpy.fft import *

def imageplot(f, str='', sbpt=[]):
    """
        Use nearest neighbor interpolation for the display.
    """
    if sbpt != []:
        plt.subplot(sbpt[0], sbpt[1], sbpt[2])
    imgplot = plt.imshow(f, interpolation='nearest')
    imgplot.set_cmap('gray')
    pylab.axis('off')
    if str != '':
        plt.title(str)

def display_image_in_actual_size(im_data):

    dpi = 80
    height, width = im_data.shape

    # What size does the figure need to be in inches to fit the image?
    figsize = width / float(dpi), height / float(dpi)

    # Create a figure of the right size with one axes that takes up the full figure
    fig = plt.figure(figsize=figsize)
    ax = fig.add_axes([0, 0, 1, 1])

    # Hide spines, ticks, etc.
    ax.axis('off')

    # Display the image.
    ax.imshow(im_data, interpolation='nearest', cmap='gray')

def FFT(x) :
    return fftshift(real(fft2(x)))

def Save( x , fname ) :
    figure()
    display_image_in_actual_size( x )
    draw()
    savefig(fname + '.png', bbox_inches='tight')

    figure()
    display_image_in_actual_size( np.log(.00001 + np.abs( FFT(x) ))  ) 
    draw()
    savefig(fname +'_FFT.png', bbox_inches='tight')


n = 512
N = n**2

name = 'nt_toolbox/data/lena.png'
x0 = load_image(name, n)

sigma = .2

y = x0 + sigma*random.standard_normal(x0.shape)
cconv = lambda a,b : real(ifft2(fft2(a)*fft2(b)))

normalize = lambda h : h/sum(h.flatten())
t = transpose( concatenate( (arange(0,n/2), arange(-n/2,0) ) ) )
[Y,X] = meshgrid(t, t)
h = lambda mu: normalize(exp(-(X**2 + Y**2)/ (2*mu**2)))

mu = 10
subplot(1,2, 1)
title('h')
subplot(1,2, 2)
title('$\hat h$');

denoise = lambda x,mu : cconv(h(mu), x)

P = 1/N * ( abs(fft2(x0))**2 )
h_w = real(ifft2(P / (P + sigma**2)))


Save( x0 , 'output/fourier/original')
Save( y , 'output/fourier/noisy')
Save( cconv(y, h(3)), 'output/fourier/gaussian_denoised')
Save( cconv(y,h_w) , 'output/fourier/wiener_denoised')

show()